// ============================================================
// Project				:	Item_PillBottle
// Author				:	Iban
// ============================================================
// Table of Contents
// 
// 1. Objects
// 2. Script
// ============================================================

// ============================================================
// Section 1 : Objects
// ============================================================
if(!isObject(ZombiePillsDebris))
{
	datablock DebrisData(ZombiePillsDebris)
	{
		shapeFile = "Add-Ons/Item_PillBottle/cap.2.dts";
		lifetime = 5.0;
		minSpinSpeed = -400.0;
		maxSpinSpeed = 200.0;
		elasticity = 0.5;
		friction = 0.2;
		numBounces = 3;
		staticOnMaxBounce = true;
		snapOnMaxBounce = false;
		fade = true;
		gravModifier = 2;
	};
}

if(!isObject(ZombiePillsItem))
{
	datablock ItemData(ZombiePillsItem)
	{
		category = "Weapon";
		className = "Weapon";
		
		shapeFile = "Add-Ons/Item_PillBottle/pills.4.dts";
		rotate = false;
		mass = 1;
		density = 0.2;
		elasticity = 0.2;
		friction = 0.6;
		emap = true;
		
		uiName = "Pills";
		iconName = "Add-Ons/Item_PillBottle/Icon_Pills";
		doColorShift = false;
		
		image = ZombiePillsImage;
		canDrop = true;
	};
}

if(!isObject(ZombiePillsImage))
{
	datablock ShapeBaseImageData(ZombiePillsImage)
	{
		shapeFile = "Add-Ons/Item_PillBottle/pills.4.dts";
		emap = true;
		mountPoint = 0;
		offset = "0 0 0";
		eyeOffset = 0;
		rotation = eulerToMatrix( "0 0 0" );
		
		className = "WeaponImage";
		item = ZombiePillsItem;
		
		armReady = true;
		
		doColorShift = ZombiePillsItem.doColorShift;
		colorShiftColor = ZombiePillsItem.colorShiftColor;
		
		casing = ZombiePillsDebris;
		shellExitDir		= "1.0 1.0 1.0";
		shellExitOffset		= "0 0 0";
		shellExitVariance	= 5;	
		shellVelocity		= 10;
		
		stateName[0]					= "Activate";
		stateSound[0]					= weaponSwitchSound;
		stateTimeoutValue[0]			= 0.15;
		stateTransitionOnTimeout[0]		= "Ready";

		stateName[1]					= "Ready";
		stateAllowImageChange[1]		= true;
		stateSequence[1]				= "Ready";
		stateTransitionOnTriggerDown[1]	= "Use";
		
		stateName[2]					= "Use";
		stateScript[2]					= "onUse";
		stateTransitionOnTriggerUp[2]	= "Done";
		
		stateName[3]					= "Done";
		stateTransitionOnAmmo[3]		= "Ready";
		stateTransitionOnNoAmmo[3]		= "UnUse";
		
		stateName[4]					= "UnUse";
		stateEjectShell[4]				= true;
		stateScript[4]					= "onUnUse";
		stateSequence[4]				= "Open";
		stateTimeoutValue[4]			= 0.5;
		stateTransitionOnTimeout[4]		= "Hack";
		stateWaitForTimeout[4]			= true;
		
		stateName[5]					= "Hack";
		stateScript[5]					= "onHack";
	};
}

// ============================================================
// Section 2 : Script
// ============================================================
function Armor::initWithdrawls(%this, %obj)
{
	if(isObject(%obj) && %obj.pseudoHealth > 0 && !isEventPending(%obj.withdrawlsSched))
	{
		%obj.pseudoHealth -= 1.5;
		
		if(%obj.pseudoHealth <= 0)
		{
			%obj.pseudoHealth = 0;
		}
		else
		{
			%obj.withdrawlsSched = %this.schedule(1500, "initWithdrawls", %obj);
		}
		
		if(isObject(%client = %obj.client))
		{
			if(isObject(ZombieFriends))
			{
				ZombieFriends.sendFriendData(%client);
			}
			
			if(%client.hasHealthBar && $HealthBar::Enabled && (!$HealthBar::MiniOnly || isObject(%client.minigame)))
			{
				%obj = %client.obj;
				
				if(isObject(%obj))
				{
					%dmgPercent = %obj.getDamageLevel() - (%obj.pseudoHealth + %obj.tf2Overheal);
					%dmgPercent = mFloatLength(%dmgPercent / %obj.getDatablock().maxDamage, 4);
					
					commandToClient(%client, 'setDamageLevel', %dmgPercent);
				}
			}
		}
	}
}

function ZombiePillsImage::onHack(%this, %obj, %slot)
{
	%obj.playThread(1, "root");
	%obj.schedule(32, "unMountImage", %slot);
	
	if(isObject(%client = %obj.client))
	{
		serverCmdUnUseTool(%client);
	}
}

function ZombiePillsImage::onUnUse(%this, %obj, %slot)
{
	%tool = %obj.tool[%obj.currTool];
	
	if(isObject(%tool) && %tool.getID() == %this.item.getID())
	{
		%armor = %obj.getDatablock();
		%damage = %obj.getDamageLevel();
		%maxDamage = %armor.maxDamage;
		%heal = %maxDamage / 2;
		
		%obj.pseudoHealth += 50;
		
		// Should NEVER overflow HP.
		// If you have 100 HP and your damage is 40, 40 - 50 = -10.
		// 50 + -10 = 40
		// 40 (pseudo) + 60 (real) = 100. You are fully healed (theorectially).
		if(%damage - %obj.pseudoHealth < 0)
		{
			%obj.pseudoHealth += (%damage - %obj.pseudoHealth);
		}
		
		%armor.initWithdrawls(%obj);
		
		%obj.playThread(2, "shiftAway");
		%obj.setWhiteOut((%maxDamage - %damage) / %maxDamage);
		
		if(isObject(%client = %obj.client))
		{
			messageClient(%client, 'MsgItemPickup', '', %obj.currTool, 0);
			
			if(%client.hasHealthBar && $HealthBar::Enabled && (!$HealthBar::MiniOnly || isObject(%client.minigame)))
			{
				%player = %client.player;
				
				if(isObject(%player))
				{
					%dmgPercent = %player.getDamageLevel() - (%player.pseudoHealth + %player.tf2Overheal);
					%dmgPercent = mFloatLength(%dmgPercent / %player.getDatablock().maxDamage, 4);
					
					commandToClient(%client, 'setDamageLevel', %dmgPercent);
				}
			}
			
			messageClient(%client, '', "\c2That should hold you.");
		}
		
		%obj.tool[%obj.currTool] = 0;
		%obj.weaponCount--;
		%obj.currTool = -1;
	}
}

function ZombiePillsImage::onUse(%this, %obj, %slot)
{
	if(%obj.getDamageLevel() - %obj.pseudoHealth > 5)
	{
		%obj.setImageAmmo(%slot, false);
	}
	else
	{
		%obj.setImageAmmo(%slot, true);
		
		%client = %obj.client;
		if(isObject(%client))
		{
			commandToClient(%client, 'centerPrint', "\c5You are not injured.", 2);
		}
	}
}

package Item_PillBottle_Package
{
	function Armor::Damage(%this, %obj, %sourceObject, %pos, %damage, %damageType)
	{
		%pseudoHealth = %obj.pseudoHealth;
		
		if(%pseudoHealth > 0)
		{
			%pseudoHealth -= %damage;
			
			if(%pseudoHealth < 0)
			{
				%damage -= %pseudoHealth;
				%pseudoHealth = 0;
			}
			else
			{
				%damage = 0;
			}
			
			%obj.pseudoHealth = %pseudoHealth;
		}
		
		if(%damage > 0)
		{
			return parent::Damage(%this, %obj, %sourceObject, %pos, %damage, %damageType);
		}
		else if(isObject(%client = %obj.client))
		{
			if(%client.hasHealthBar && $HealthBar::Enabled && (!$HealthBar::MiniOnly || isObject(%client.minigame)))
			{
				%dmgPercent = %obj.getDamageLevel() - (%obj.pseudoHealth + %obj.tf2Overheal);
				%dmgPercent = mFloatLength(%dmgPercent / %obj.getDatablock().maxDamage, 4);
				
				commandToClient(%client, 'setDamageLevel', %dmgPercent);
			}
		}
	}
};
activatePackage(Item_PillBottle_Package);